@@ -1,10 +1,28 @@
 import sys
-from typing import Optional, Any, List, Tuple, Set, Callable, Dict
+import os
+from typing import (
+    Optional,
+    Any,
+    List,
+    Tuple,
+    Set,
+    Callable,
+    Dict,
+    Union,
+    Iterator,
+    Sequence,
+)
 
 from robocorp_ls_core.cache import instance_cache
 from robocorp_ls_core.constants import NULL
 from robocorp_ls_core.lsp import CompletionItemTypedDict, MarkupContentTypedDict
-from robocorp_ls_core.protocols import IMonitor, Sentinel, IConfig, IDocumentSelection
+from robocorp_ls_core.protocols import (
+    IMonitor,
+    Sentinel,
+    IConfig,
+    IDocumentSelection,
+    IWorkspace,
+)
 from robocorp_ls_core.robotframework_log import get_logger
 from robotframework_ls.impl.protocols import (
     IRobotDocument,
@@ -16,43 +34,28 @@ from robotframework_ls.impl.protocols import (
     KeywordUsageInfo,
     CompletionType,
     IResourceImportNode,
+    ICompletionContextDependencyGraph,
+    IRobotToken,
+    INode,
+    IVariableImportNode,
+    VarTokenInfo,
+    IVariablesFromArgumentsFileLoader,
 )
 from robotframework_ls.impl.robot_workspace import RobotDocument
+from robocorp_ls_core import uris
+import itertools
+from functools import partial
+import typing
+from robotframework_ls.impl.variables_from_arguments_file import (
+    VariablesFromArgumentsFileLoader,
+)
 
 
 log = get_logger(__name__)
 
 
 class _Memo(object):
-    def __init__(self):
-        self.clear()
-
-    def clear(self):
-        self._followed_imports_variables = {}
-        self._followed_imports = {}
-        self._completed_libraries = {}
-
-    def follow_import(self, uri: str) -> bool:
-        if uri not in self._followed_imports:
-            self._followed_imports[uri] = True
-            return True
-
-        return False
-
-    def follow_import_variables(self, uri: str) -> bool:
-        if uri not in self._followed_imports_variables:
-            self._followed_imports_variables[uri] = True
-            return True
-
-        return False
-
-    def complete_for_library(self, library_name: str, alias: Optional[str]) -> bool:
-        key = (library_name, alias)
-        if key not in self._completed_libraries:
-            self._completed_libraries[key] = True
-            return True
-
-        return False
+    pass
 
 
 class BaseContext(object):
@@ -94,20 +97,14 @@ class CompletionContext(object):
         doc,
         line=Sentinel.SENTINEL,
         col=Sentinel.SENTINEL,
-        workspace=None,
-        config=None,
-        memo=None,
+        workspace: Optional[IWorkspace] = None,
+        config: Optional[IConfig] = None,
+        memo: Optional[_Memo] = None,
         monitor: Optional[IMonitor] = NULL,
+        variables_from_arguments_files_loader: Sequence[
+            IVariablesFromArgumentsFileLoader
+        ] = (),
     ) -> None:
-        """
-        :param robocorp_ls_core.workspace.Document doc:
-        :param int line:
-        :param int col:
-        :param RobotWorkspace workspace:
-        :param robocorp_ls_core.config.Config config:
-        :param _Memo memo:
-        """
-
         if col is Sentinel.SENTINEL or line is Sentinel.SENTINEL:
             assert (
                 col is Sentinel.SENTINEL
@@ -131,7 +128,7 @@ class CompletionContext(object):
 
         self._doc = doc
         self._sel = sel
-        self._workspace = workspace
+        self._workspace = typing.cast(Optional[IRobotWorkspace], workspace)
         self._config = config
         self._memo = memo
         self._original_ctx: Optional[CompletionContext] = None
@@ -141,6 +138,9 @@ class CompletionContext(object):
         self._id_to_compute_documentation: Dict[
             int, Callable[[], MarkupContentTypedDict]
         ] = {}
+        self.variables_from_arguments_files_loader = (
+            variables_from_arguments_files_loader
+        )
 
     def assign_documentation_resolve(
         self,
@@ -195,6 +195,7 @@ class CompletionContext(object):
             config=self._config,
             memo=self._memo,
             monitor=self._monitor,
+            variables_from_arguments_files_loader=self.variables_from_arguments_files_loader,
         )
         ctx._original_ctx = self
         return ctx
@@ -208,6 +209,7 @@ class CompletionContext(object):
             config=self._config,
             memo=self._memo,
             monitor=self._monitor,
+            variables_from_arguments_files_loader=self.variables_from_arguments_files_loader,
         )
         ctx._original_ctx = self
         return ctx
@@ -242,6 +244,7 @@ class CompletionContext(object):
 
     @property
     def workspace(self) -> IRobotWorkspace:
+        assert self._workspace
         return self._workspace
 
     @instance_cache
@@ -340,7 +343,10 @@ class CompletionContext(object):
         return tuple(ast_utils.iter_variables(ast))
 
     @instance_cache
-    def get_current_variable(self, section=None) -> Optional[TokenInfo]:
+    def get_current_variable(self, section=None) -> Optional[VarTokenInfo]:
+        """
+        Provides the current variable token. Note that it won't include '{' nor '}'.
+        """
         from robotframework_ls.impl import ast_utils
 
         if section is None:
@@ -373,7 +379,7 @@ class CompletionContext(object):
         return tuple(ret)
 
     @instance_cache
-    def get_variable_imports(self):
+    def get_variable_imports(self) -> Tuple[INode, ...]:
         from robotframework_ls.impl import ast_utils
 
         ast = self.get_ast()
@@ -383,112 +389,98 @@ class CompletionContext(object):
                 ret.append(resource.node)
         return tuple(ret)
 
-    def token_value_resolving_variables(self, token):
-        from robotframework_ls.impl import ast_utils
+    def token_value_resolving_variables(self, token: Union[str, IRobotToken]) -> str:
+        from robotframework_ls.impl.variable_resolve import ResolveVariablesContext
 
-        if isinstance(token, str):
-            token = ast_utils.create_token(token)
+        return ResolveVariablesContext(
+            self.config, self._doc.path
+        ).token_value_resolving_variables(token)
 
-        try:
-            tokenized_vars = ast_utils.tokenize_variables(token)
-        except:
-            return token.value  # Unable to tokenize
-        parts = []
-        for v in tokenized_vars:
-            if v.type == v.NAME:
-                parts.append(str(v))
+    def token_value_and_unresolved_resolving_variables(
+        self, token: IRobotToken
+    ) -> Tuple[str, Tuple[IRobotToken, ...]]:
+        from robotframework_ls.impl.variable_resolve import ResolveVariablesContext
 
-            elif v.type == v.VARIABLE:
-                # Resolve variable from config
-                initial_v = v = str(v)
-                if v.startswith("${") and v.endswith("}"):
-                    v = v[2:-1]
-                    parts.append(self._convert_robot_variable(v, initial_v))
-                elif v.startswith("%{") and v.endswith("}"):
-                    v = v[2:-1]
-                    parts.append(self._convert_environment_variable(v, initial_v))
-                else:
-                    log.info("Cannot resolve variable: %s", v)
-                    parts.append(v)  # Leave unresolved.
-
-        joined_parts = "".join(parts)
-        return joined_parts
+        return ResolveVariablesContext(
+            self.config, self._doc.path
+        ).token_value_and_unresolved_resolving_variables(token)
 
     @instance_cache
-    def get_resource_import_as_doc(self, resource_import) -> Optional[IRobotDocument]:
-        from robocorp_ls_core import uris
-        import os.path
+    def get_resource_import_as_doc(
+        self, resource_import: INode
+    ) -> Optional[IRobotDocument]:
+        from robot.api import Token
         from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_PYTHONPATH
 
-        ws = self._workspace
+        ws = self.workspace
+        token = resource_import.get_token(Token.NAME)
+        if token is not None:
+            name_with_resolved_vars = self.token_value_resolving_variables(token)
 
-        for token in resource_import.tokens:
-            if token.type == token.NAME:
-
-                name_with_resolved_vars = self.token_value_resolving_variables(token)
-
-                if not os.path.isabs(name_with_resolved_vars):
-                    # It's a relative resource, resolve its location based on the
-                    # current file.
-                    check_paths = [
-                        os.path.normpath(
-                            os.path.join(
-                                os.path.dirname(self.doc.path), name_with_resolved_vars
-                            )
+            if not os.path.isabs(name_with_resolved_vars):
+                # It's a relative resource, resolve its location based on the
+                # current file.
+                check_paths = [
+                    os.path.normpath(
+                        os.path.join(
+                            os.path.dirname(self.doc.path), name_with_resolved_vars
                         )
-                    ]
-                    config = self.config
-                    if config is not None:
-                        for additional_pythonpath_entry in config.get_setting(
-                            OPTION_ROBOT_PYTHONPATH, list, []
-                        ):
-                            check_paths.append(
-                                os.path.normpath(
-                                    os.path.join(
-                                        additional_pythonpath_entry,
-                                        name_with_resolved_vars,
-                                    )
-                                )
-                            )
-
-                    for path in sys.path:
+                    )
+                ]
+                config = self.config
+                if config is not None:
+                    for additional_pythonpath_entry in config.get_setting(
+                        OPTION_ROBOT_PYTHONPATH, list, []
+                    ):
                         check_paths.append(
                             os.path.normpath(
                                 os.path.join(
-                                    path,
+                                    additional_pythonpath_entry,
                                     name_with_resolved_vars,
                                 )
                             )
                         )
 
-                    # Make the entries in the list unique (keeping order).
-                    seen: Set[str] = set()
-                    add_it_and_return_none = seen.add
-                    check_paths = [
-                        x
-                        for x in check_paths
-                        if x not in seen and not add_it_and_return_none(x)
-                    ]
+                for path in sys.path:
+                    check_paths.append(
+                        os.path.normpath(
+                            os.path.join(
+                                path,
+                                name_with_resolved_vars,
+                            )
+                        )
+                    )
 
-                else:
-                    check_paths = [name_with_resolved_vars]
+                # Make the entries in the list unique (keeping order).
+                seen: Set[str] = set()
+                add_it_and_return_none = seen.add
+                check_paths = [
+                    x
+                    for x in check_paths
+                    if x not in seen and not add_it_and_return_none(x)
+                ]
 
-                for resource_path in check_paths:
-                    doc_uri = uris.from_fs_path(resource_path)
-                    resource_doc = ws.get_document(doc_uri, accept_from_file=True)
-                    if resource_doc is None:
-                        continue
-                    return resource_doc
+            else:
+                check_paths = [name_with_resolved_vars]
 
-                log.info(
-                    "Unable to find: %s (checked paths: %s)",
-                    name_with_resolved_vars,
-                    check_paths,
-                )
+            for resource_path in check_paths:
+                doc_uri = uris.from_fs_path(resource_path)
+                resource_doc = ws.get_document(doc_uri, accept_from_file=True)
+                if resource_doc is None:
+                    continue
+                return typing.cast(IRobotDocument, resource_doc)
+
+            log.info(
+                "Unable to find: %s (checked paths: %s)",
+                name_with_resolved_vars,
+                check_paths,
+            )
 
         return None
 
-    def get_variable_import_as_doc(self, variables_import) -> Optional[IRobotDocument]:
+    def get_variable_import_as_doc(
+        self, variables_import: INode
+    ) -> Optional[IRobotDocument]:
         return self.get_resource_import_as_doc(variables_import)
 
     @instance_cache
@@ -506,89 +498,81 @@ class CompletionContext(object):
         return tuple(ret)
 
     @instance_cache
-    def get_variable_imports_as_docs(self) -> Tuple[IRobotDocument, ...]:
+    def get_resource_inits_as_docs(self) -> Tuple[IRobotDocument, ...]:
+        doc = self.doc
+        path = doc.path
+        if not path:
+            return ()
+
+        if not os.path.isabs(path):
+            # i.e.: We can't deal with untitled...
+            return ()
+
+        parent_dir_path = os.path.dirname(path)
+
         ret: List[IRobotDocument] = []
+        ws = self.workspace
+        folder_root_paths = ws.get_folder_paths()
+
+        for root_path in folder_root_paths:
+            if parent_dir_path.startswith(root_path):
+                stop_condition = lambda parent_dir_path: parent_dir_path == root_path
+                break
+
+        else:
+            # We're dealing with a resource out of the workspace root,
+            # so, let's stop at max range.
+            next_i: "partial[int]" = partial(next, itertools.count())
+            stop_condition = lambda parent_dir_path: next_i() >= 6
+
+        initial_parent_path = parent_dir_path
+
+        def iter_inits():
+            parent_dir_path = initial_parent_path
+            while True:
+                yield os.path.join(parent_dir_path, "__init__.robot")
+
+                if stop_condition(parent_dir_path):
+                    break
+
+                initial_len = len(parent_dir_path)
+                parent_dir_path = os.path.dirname(parent_dir_path)
+                if initial_len == len(parent_dir_path) or not len(parent_dir_path):
+                    break
+
+        for resource_path in iter_inits():
+            doc_uri = uris.from_fs_path(resource_path)
+            resource_doc = ws.get_document(doc_uri, accept_from_file=True)
+            if resource_doc is not None:
+                ret.append(typing.cast(IRobotDocument, resource_doc))
+        return tuple(ret)
+
+    def iter_dependency_and_init_resource_docs(
+        self, dependency_graph
+    ) -> Iterator[IRobotDocument]:
+        visited = set()
+        for resource_doc in itertools.chain(
+            (d[1] for d in dependency_graph.iter_all_resource_imports_with_docs()),
+            iter(self.get_resource_inits_as_docs()),
+        ):
+            if resource_doc is not None:
+                if resource_doc.uri not in visited:
+                    visited.add(resource_doc.uri)
+                    yield resource_doc
+
+    @instance_cache
+    def get_variable_imports_as_docs(
+        self,
+    ) -> Tuple[Tuple[IVariableImportNode, Optional[IRobotDocument]], ...]:
+        ret: List[Tuple[IResourceImportNode, Optional[IRobotDocument]]] = []
 
         variable_imports = self.get_variable_imports()
         for variable_import in variable_imports:
             variable_doc = self.get_variable_import_as_doc(variable_import)
-            if variable_doc is not None:
-                ret.append(variable_doc)
+            ret.append((variable_import, variable_doc))
 
         return tuple(ret)
 
-    def _resolve_builtin(self, var_name, value_if_not_found, log_info):
-        from robotframework_ls.impl.robot_constants import BUILTIN_VARIABLES_RESOLVED
-
-        ret = BUILTIN_VARIABLES_RESOLVED.get(var_name, Sentinel.SENTINEL)
-        if ret is Sentinel.SENTINEL:
-            if var_name == "CURDIR":
-                import os
-
-                return os.path.dirname(self._doc.path)
-            log.info(*log_info)
-            return value_if_not_found
-        return ret
-
-    def _resolve_environment_variable(self, var_name, value_if_not_found, log_info):
-        import os
-
-        ret = os.environ.get(var_name, Sentinel.SENTINEL)
-        if ret is Sentinel.SENTINEL:
-            log.info(*log_info)
-            return value_if_not_found
-        return ret
-
-    def _convert_robot_variable(self, var_name, value_if_not_found):
-        from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_VARIABLES
-
-        if self.config is None:
-            value = self._resolve_builtin(
-                var_name,
-                value_if_not_found,
-                (
-                    "Config not available while trying to convert robot variable: %s",
-                    var_name,
-                ),
-            )
-        else:
-            robot_variables = self.config.get_setting(OPTION_ROBOT_VARIABLES, dict, {})
-            value = robot_variables.get(var_name, Sentinel.SENTINEL)
-            if value is Sentinel.SENTINEL:
-                value = self._resolve_builtin(
-                    var_name,
-                    value_if_not_found,
-                    ("Unable to find robot variable: %s", var_name),
-                )
-
-        value = str(value)
-        return value
-
-    def _convert_environment_variable(self, var_name, value_if_not_found):
-        from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_PYTHON_ENV
-
-        if self.config is None:
-            value = self._resolve_environment_variable(
-                var_name,
-                value_if_not_found,
-                (
-                    "Config not available while trying to convert environment variable: %s",
-                    var_name,
-                ),
-            )
-        else:
-            robot_env_vars = self.config.get_setting(OPTION_ROBOT_PYTHON_ENV, dict, {})
-            value = robot_env_vars.get(var_name, Sentinel.SENTINEL)
-            if value is Sentinel.SENTINEL:
-                value = self._resolve_environment_variable(
-                    var_name,
-                    value_if_not_found,
-                    ("Unable to find environment variable: %s", var_name),
-                )
-
-        value = str(value)
-        return value
-
     @instance_cache
     def get_current_keyword_definition(self) -> Optional[IKeywordDefinition]:
         """
@@ -653,11 +637,19 @@ class CompletionContext(object):
             definitions = find_keyword_definition(
                 cp, TokenInfo(usage_info.stack, usage_info.node, usage_info.token)
             )
-            if definitions and len(definitions) >= 1:
+            if definitions:
                 definition: IKeywordDefinition = next(iter(definitions))
                 return definition, usage_info
         return None
 
+    @instance_cache
+    def collect_dependency_graph(self) -> ICompletionContextDependencyGraph:
+        from robotframework_ls.impl.completion_context_dependency_graph import (
+            CompletionContextDependencyGraph,
+        )
+
+        return CompletionContextDependencyGraph.from_completion_context(self)
+
     def __typecheckself__(self) -> None:
         from robocorp_ls_core.protocols import check_implements
 
