@@ -22,7 +22,7 @@ import threading
 from contextlib import contextmanager
 from robocorp_ls_core.robotframework_log import get_logger
 from robocorp_ls_core.options import DEFAULT_TIMEOUT
-from typing import TypeVar, Any, Callable
+from typing import TypeVar, Any, Callable, Tuple
 from robocorp_ls_core.jsonrpc.exceptions import JsonRpcRequestCancelled
 from functools import lru_cache
 
@@ -354,7 +354,7 @@ def before(obj, method_name, callback):
         setattr(obj, method_name, original_method)
 
 
-def check_min_version(version, min_version):
+def check_min_version(version: str, min_version: Tuple[int, int]) -> bool:
     """
     :param version:
         This is the version of robotframework.
@@ -366,11 +366,11 @@ def check_min_version(version, min_version):
         True if version >= min_versiond and False otherwise.
     """
     try:
-        version = tuple(int(x) for x in version.split("."))
+        v = tuple(int(x) for x in version.split("."))
     except:
         return False
 
-    return version >= min_version
+    return v >= min_version
 
 
 def wait_for_condition(condition, msg=None, timeout=DEFAULT_TIMEOUT, sleep=1 / 20.0):
