@@ -359,9 +359,14 @@ class Range(_Base):
 
 
 class TextDocumentContentChangeEvent(_Base):
-    def __init__(self, range, rangeLength, text):
+    def __init__(
+        self, range: Optional[RangeTypedDict], rangeLength: Optional[int], text: str
+    ):
+        """
+        :param rangeLength: Deprecated
+        """
         self.range = range
-        self.rangeLength = rangeLength
+        self.rangeLength = rangeLength  # Deprecated
         self.text = text
 
 
@@ -464,7 +469,7 @@ class TextDocumentPositionParamsTypedDict(TypedDict, total=False):
 
 
 class MarkupContentTypedDict(TypedDict):
-    kind: str  # "plaintext" | "markdown"
+    kind: str  # "plaintext" | "markdown" # see: MarkupKind
     value: str
 
 
@@ -590,7 +595,7 @@ class CompletionItemTypedDict(TypedDict, total=False):
 
 class HoverTypedDict(TypedDict, total=False):
     contents: MarkupContentTypedDict
-    range: RangeTypedDict  # Optional
+    range: Optional[RangeTypedDict]
 
 
 class DocumentHighlightTypedDict(TypedDict, total=False):
